#ifndef __IS_PTR_H__
#define __IS_PTR_H__

#include "pCOM.h"
/*
 *
 * Smart pointer IPtr template.
 * From Chapter 9 of "Inside COM" by Dale Rogerson (Microsoft Press)
 * 
 */
 
template <class T, const IID* piid> class IPtr
{
   public:
   //Constructor
   IPtr()
   {
      m_pI = NULL;
   };
   
   //Destructor
   ~IPtr()
   {
      Release();
   }
   
   
   void Release()
   {
      if(m_pI != NULL)
      {
         T* pOld = m_pI;
         m_pI = NULL;
         pOld->Release();
      }
   }
   
   //Conversion; ex (IString *)mySmartPtr
   operator T*() {return m_pI;}
    
   //Pointer operations
   T& operator*() {return *m_pI;}
   T** operator&() {return &m_pI;}
   T* operator->() {return m_pI;}
   
   //Assignment from the same interface
   T* operator=(T* pI)
   {
      if(m_pI != pI)
      {
         IUnknown* pOld = m_pI; //Save current value.
         m_pI = pI;             //Assign new value
         
         if(m_pI != NULL)
            m_pI->AddRef();    
            
         if(pOld != NULL)
            pOld->Release();  //Release old interface
      }
      
      return m_pI;
   }
   
   //Assignment from another interface
   T* operator=(IUnknown* pI)
   {
      IUnknown* pOld = m_pI;  //Save current value
      m_pI = NULL;
      
      //Query for correct interface
      if(pI != NULL)
      {
         HRESULT hr = pI->QueryInterface(*piid, (void**)&m_pI);
         assert(SUCCEEDED(hr) && (m_pI != NULL));
      }
      
      if(pOld != NULL)
         pOld->Release();   //Release old pointer
         
      return m_pI;
   }
   
   //Boolean functions
   BOOL operator!(){ return (m_pI == NULL) ? TRUE : FALSE;}
   
   //GUID
   const IID& iid() {return *piid;}
   
   private:
   //pointer variable
   T* m_pI;
      
};
 
#endif